<section class="content-header">
  <h1>
    {% block section_title %}{% endblock %}
    <small>{% block section_subtitle %}{% endblock %}</small>
  </h1>
</section>

<section class="content">
    <div class="container-fluid">{% block section_content %}{% endblock %}</div>
</section>