{% extends 'core_main_app/_render/admin/theme/tools/modal.html' %}
{% load stripjs %}

{% block modal_id %}login-modal-id{% endblock %}
{% block modal_title %}Warning{% endblock %}
{% block modal_body %}
<p class="word-wrap">{{ data.login_message | stripjs }}</p>
{% endblock %}

{% block modal_footer %}
    <button id="btn-login-cancel" class="btn btn-danger pull-left"><i class="fa fa-times"></i> Cancel</button>
    <button id="btn-login-yes" class="btn btn-secondary"><i class="fa fa-check"></i> Accept</button>
{% endblock %}
