# Generated by Django 2.2.13 on 2020-09-01 13:41

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ("core_main_app", "0002_site_update"),
    ]

    operations = [
        migrations.AlterModelOptions(
            name="main",
            options={
                "default_permissions": (),
                "permissions": (
                    ("publish_data", "Can publish data"),
                    ("publish_blob", "Can publish blob"),
                ),
                "verbose_name": "core_main_app",
            },
        ),
    ]
