/**
 * CSS for tables
 */
.fixed-table {
    table-layout: fixed;
    width: 100%;
}

.fixed-table th,td {
    word-wrap: break-word;
}

.fixed-table tr>td>i{
    float: left;
    padding-right: 0.5em;
    padding-top: 0.2em;
}

.col-width-75 {
    width: 75%:
}

.col-width-25 {
    width: 25%;
}

.template-name {
    width: 75%;
}

.table-col-3 th {
    width: 33%;
}

.table-col-4 th {
    width: 25%;
}

/**
 * Table pagination
**/
.pagination-container {
    text-align: center;
}