<section class="content">
    <div class="row">
        <div class="col-md-12">
            {% block list_includes %}
            {% include 'core_main_app/common/templates/versions/available.html' %}
            {% endblock %}
        </div>
    </div>
</section>
