{% extends "core_main_app/_render/admin/theme/tools/section.html" %}
{% load staticfiles %}

{% block section_title %}Data detail{% endblock %}

{% block section_content %}
<div class="row">
    <div class="col-md-12">
        {% include 'core_main_app/admin/data/detail_box.html' %}
    </div>
</div>
{% endblock %}