{% extends 'core_main_app/_render/admin/theme/tools/modal.html' %}

{% block modal_title %}Shareable link{% endblock %}

{% block modal_body %}
    <div class="alert alert-danger" id="banner_set_name_query_errors" style="display: none;">
            <a href="#" class="close" data-hide="alert" aria-label="close">&times;</a>
            <h4><i class="fa fa-exclamation-circle"></i> Error</h4>
            <div id="set_name_query_errors"></div>
    </div>
    <div>
        <input type="text" id="{% block modal_input_id %}{% endblock %}" class="shared-data"
           disabled="disabled"/>
    </div>

    {% block modal_extra_content %}{% endblock %}

{% endblock %}

{% block modal_footer %}
    <div class="w-100 d-flex flex-row-reverse">
        <button id="{% block modal_submit_id %}{% endblock %}" class="btn btn-primary">
            <i class="fas fa-copy"></i> Copy and close
        </button>
    </div>
{% endblock %}