let loadDataUrlBase = "{% url 'core_main_app_rest_data_query' %}";
let migrationUrlBase = "{% url 'core_main_app_rest_data_migrate' pk='placeholder_id' %}";
let taskBaseUrl = "{% url 'core_main_app_rest_data_migration_task_progress' task_id='placeholder_id' %}";