""" Constants
"""

MARKDOWN_UNSAFE = 0
MARKDOWN_GENERATION_FAILED = 1

MARKDOWN_ERRORS = [
    "Unprotected script tags have been detected. Please remove them to be able to save your work!",
    "HTML is not generated properly, please check your Markdown and HTML syntax!",
]

UNKNOWN_ERROR = (
    "An unknown error occured, please contact your administrator for more information."
)
