{% extends 'core_main_app/_render/admin/theme/tools/modal.html' %}

{% block modal_id %}delete-object-modal{% endblock %}
{% block modal_title %}Delete {{ data.object_name }}{% endblock %}

{% block modal_body %}
{% endblock %}

{% block modal_footer %}
<button class="btn btn-secondary pull-left" data-dismiss="modal"><i class="fa fa-times"></i>
    Cancel
</button>
<button id="delete-object-save" class="btn btn-danger">
    <i class="fa fa-trash"></i> Delete
</button>
{% endblock %}