{% extends 'core_main_app/_render/admin/theme/tools/modal.html' %}

{% block modal_id %}add-user-modal{% endblock %}
{% block modal_title %} Give user rights to workspace {% endblock %}

{% block modal_body %}
<div class="alert alert-danger" id="banner_rights_errors" style="display: none;">
    <h4><i class="fa fa-exclamation-circle fa-1x"></i> Error</h4>
    <div id="form_edit_rights_errors">
    </div>
</div>

Select all users you want to give rights to the workspace:
<div id="add-user-form">
</div>
<div id="list-rights">
    <label class="switch">
        <input type="checkbox" id="read">
        <span class="slider"></span>
    </label> Read
    <br/>
    <label class="switch">
        <input type="checkbox" id="write">
        <span class="slider"></span>
    </label> Write
</div>
{% endblock %}

{% block modal_footer %}
<a class="btn btn-secondary pull-left" data-dismiss="modal" href="#"><i class="fa fa-times"></i> Cancel</a>
<a id="add-user-yes" class="btn btn-secondary" href="#"><i class="fa fa-check"></i> Add</a>
{% endblock %}

