<div>
    <button id="btn-previous-page" type="button" class="btn btn-secondary float-right">
        <i class="fa fa-arrow-left"></i> Back to previous
    </button>
    {% if data.share_pid_button %}
        {% include 'core_linked_records_app/user/sharing/data_detail/button.html' %}
    {% endif %}
    {% if 'core_tsne_app' in INSTALLED_APPS %}
        <a style="margin-top: 1em;" class="btn btn-primary float-right"
           href="{% url 'core_tsne_app_data' data.data.id %}">
            view on t-sne map
        </a>
    {% endif %}

        <div class="row float-right mr-1"  {% if not can_display_selector %} style="display:none" {% endif %}>
            <div class="form-group ">
                <select class="selectpicker form-control" id="xslt-selector" >
                   {% if not template_xsl_rendering.default_detail_xslt %}  <option value={{None}}>Default</option>{% endif %}
                   {% for xslt in template_xsl_rendering.list_detail_xslt %}
                        {% if xslt.id == xslt_id %}
                            <option value="{{xslt.id}}" selected >{{xslt.name}} {% if xslt == template_xsl_rendering.default_detail_xslt %} (default) {% endif %}</option>
                        {% else %}
                            <option value="{{xslt.id}}" >{{xslt.name}} {% if xslt == template_xsl_rendering.default_detail_xslt %} (default) {% endif %}</option>
                        {% endif %}
                   {% endfor %}
                </select>
            </div>
        </div>

</div>
