{% extends 'core_main_app/_render/admin/theme/tools/modal.html' %}

{% block modal_title %}Shareable links{% endblock %}

{% block modal_body %}
<textarea id="{% block modal_input_id %}{% endblock %}" class="textarea-lg" disabled="true"></textarea>
{% endblock %}

{% block modal_footer %}
<div class="w-100 d-flex flex-row-reverse">
    <button id="{% block modal_submit_id %}{% endblock %}" class="btn btn-primary">
        <i class="fas fa-copy"></i> Copy and close
    </button>
</div>
{% endblock %}