<div class="alert alert-warning">
    The uploaded schema contains one or more import/include elements. Some references could not be loaded. Please check
    that the resource is accessible (not on your local file system, or on a server down). You can choose files from the
    database to resolve each import/include.
</div>
<table id="dependencies" class="table table-bordered table-striped table-hover">
    <tr>
        <th>Import/Include</th>
        <th>Schema Location</th>
        <th>Dependency</th>
    </tr>

    {% for import in imports %}
    <tr>
        <td>Import</td>
        <td class="schemaLocation">
            <!--<textarea readonly="readonly" class="schemaLocation">-->
            {{ import.attrib.schemaLocation }}
            <!--</textarea>-->
        </td>
        <td>{{ dependencies }}</td>
    </tr>
    {% endfor %}

    {% for include in includes %}
    <tr>
        <td>Include</td>
        <td class="schemaLocation">
            <!--<textarea readonly="readonly" class="schemaLocation">-->
            {{ include.attrib.schemaLocation }}
            <!--</textarea>-->
        </td>
        <td>{{ dependencies }}</td>
    </tr>
    {% endfor %}
</table>
<div id="xsd_content" style="display: none;">{{ xsd_content }}</div>
<div id="filename" style="display: none;">{{ filename }}</div>
<span class='resolve'>
    <a class="btn btn-primary float-right" href="#"><i class="fa fa-save"></i> Save</a>
</span>
<div id='errorDependencies' style="color:red;"></div>